# Copyright (c) 2021 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_policy import policy as base_policy
from oslo_utils import uuidutils

from neutron import policy
from neutron.tests.unit.conf.policies import test_base as base


class QosPolicyAPITestCase(base.PolicyBaseTestCase):

    def setUp(self):
        super().setUp()
        self.target = {'project_id': self.project_id}
        self.alt_target = {'project_id': self.alt_project_id}


class SystemAdminQosPolicyTests(QosPolicyAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'get_policy', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'get_policy', self.alt_target)

    def test_get_policy_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'get_policy:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'get_policy:tags', self.alt_target)

    def test_create_policy(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_policy', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_policy', self.alt_target)

    def test_create_policy_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_policy:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'create_policy:tags',
            self.alt_target)

    def test_update_policy(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_policy', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_policy', self.alt_target)

    def test_update_policy_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_policy:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'update_policy:tags',
            self.alt_target)

    def test_delete_policy(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_policy', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_policy', self.alt_target)

    def test_delete_policy_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_policy:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce, self.context, 'delete_policy:tags',
            self.alt_target)


class SystemMemberQosPolicyTests(SystemAdminQosPolicyTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosPolicyTests(SystemMemberQosPolicyTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosPolicyTests(QosPolicyAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_policy', self.alt_target))

    def test_get_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy:tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_policy:tags', self.alt_target))

    def test_create_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_policy', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'create_policy', self.alt_target))

    def test_create_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_policy:tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'create_policy:tags',
                           self.alt_target))

    def test_update_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_policy', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_policy', self.alt_target))

    def test_update_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_policy:tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_policy:tags',
                           self.alt_target))

    def test_delete_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy', self.alt_target))

    def test_delete_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy:tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy:tags',
                           self.alt_target))


class ProjectManagerQosPolicyTests(AdminQosPolicyTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy', self.alt_target)

    def test_get_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy:tags',
            self.alt_target)

    def test_create_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_policy', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy', self.alt_target)

    def test_create_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_policy:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy:tags',
            self.alt_target)

    def test_update_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_policy', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy', self.alt_target)

    def test_update_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_policy:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy:tags',
            self.alt_target)

    def test_delete_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy', self.alt_target)

    def test_delete_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_policy:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy:tags',
            self.alt_target)


class ProjectMemberQosPolicyTests(ProjectManagerQosPolicyTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_get_policy(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy', self.alt_target)

    def test_get_policy_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_policy:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy:tags',
            self.alt_target)

    def test_create_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy', self.alt_target)

    def test_create_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy:tags',
            self.alt_target)

    def test_update_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy', self.alt_target)

    def test_update_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy:tags',
            self.alt_target)

    def test_delete_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy', self.alt_target)

    def test_delete_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy:tags',
            self.alt_target)


class ProjectReaderQosPolicyTests(ProjectMemberQosPolicyTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosPolicyTests(QosPolicyAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy', self.target)

    def test_get_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'get_policy:tags', self.target)

    def test_create_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy', self.target)

    def test_create_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'create_policy:tags', self.target)

    def test_update_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy', self.target)

    def test_update_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'update_policy:tags', self.target)

    def test_delete_policy(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy', self.target)

    def test_delete_policy_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce, self.context, 'delete_policy:tags', self.target)


class QosRuleTypeAPITestCase(base.PolicyBaseTestCase):

    def setUp(self):
        super().setUp()
        self.target = {}


class SystemAdminQosRuleTypeTests(QosRuleTypeAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_rule_type(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_rule_type', self.target)


class SystemMemberQosRuleTypeTests(SystemAdminQosRuleTypeTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosRuleTypeTests(SystemMemberQosRuleTypeTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosRuleTypeTests(QosRuleTypeAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_rule_type(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_rule_type', self.target))


class ProjectManagerQosRuleTypeTests(AdminQosRuleTypeTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx


class ProjectMemberQosRuleTypeTests(ProjectManagerQosRuleTypeTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx


class ProjectReaderQosRuleTypeTests(ProjectMemberQosRuleTypeTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosRuleTypeTests(QosRuleTypeAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_rule_type(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_rule_type', self.target)


class QosRulesAPITestCase(base.PolicyBaseTestCase):

    def setUp(self):
        super().setUp()
        self.qos_policy = {
            'id': uuidutils.generate_uuid(),
            'project_id': self.project_id}
        self.alt_qos_policy = {
            'id': uuidutils.generate_uuid(),
            'project_id': self.alt_project_id}
        self.target = {
            'policy_id': self.qos_policy['id'],
            'ext_parent_policy_id': self.qos_policy['id']}
        self.alt_target = {
            'policy_id': self.alt_qos_policy['id'],
            'ext_parent_policy_id': self.alt_qos_policy['id']}

        self.plugin_mock = mock.Mock()
        mock.patch(
            'neutron_lib.plugins.directory.get_plugin',
            return_value=self.plugin_mock).start()


class SystemAdminQosBandwidthLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_bandwidth_limit_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_bandwidth_limit_rule',
                self.target)

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_bandwidth_limit_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_bandwidth_limit_rule',
                self.alt_target)

    def test_create_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_bandwidth_limit_rule',
            self.alt_target)

    def test_update_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_bandwidth_limit_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_bandwidth_limit_rule',
            self.alt_target)

    def test_delete_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_bandwidth_limit_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_bandwidth_limit_rule',
            self.alt_target)


class SystemMemberQosBandwidthLimitRuleTests(
        SystemAdminQosBandwidthLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosBandwidthLimitRuleTests(
        SystemMemberQosBandwidthLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosBandwidthLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_bandwidth_limit_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_bandwidth_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_bandwidth_limit_rule',
                               self.alt_target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_bandwidth_limit_rule',
                               self.alt_target))

    def test_create_policy_bandwidth_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_bandwidth_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_bandwidth_limit_rule',
                           self.alt_target))

    def test_update_policy_bandwidth_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_bandwidth_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_bandwidth_limit_rule',
                           self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_bandwidth_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_bandwidth_limit_rule',
                           self.alt_target))

    def test_delete_policy_bandwidth_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_bandwidth_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_bandwidth_limit_rule',
                           self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_alias_bandwidth_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_alias_bandwidth_limit_rule',
                           self.alt_target))


class ProjectManagerQosBandwidthLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_bandwidth_limit_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_bandwidth_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_bandwidth_limit_rule',
                self.alt_target)

            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_bandwidth_limit_rule',
                self.alt_target)

    def test_create_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'create_policy_bandwidth_limit_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'create_policy_bandwidth_limit_rule',
                self.alt_target)

    def test_update_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'update_policy_bandwidth_limit_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'update_alias_bandwidth_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_policy_bandwidth_limit_rule',
                self.alt_target)

            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_alias_bandwidth_limit_rule',
                self.alt_target)

    def test_delete_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_policy_bandwidth_limit_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_alias_bandwidth_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_policy_bandwidth_limit_rule',
                self.alt_target)

            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_alias_bandwidth_limit_rule',
                self.alt_target)


class ProjectMemberQosBandwidthLimitRuleTests(
        ProjectManagerQosBandwidthLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_create_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_bandwidth_limit_rule',
            self.alt_target)

    def test_update_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_bandwidth_limit_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_bandwidth_limit_rule',
            self.alt_target)

    def test_delete_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_bandwidth_limit_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_bandwidth_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_bandwidth_limit_rule',
            self.alt_target)


class ProjectReaderQosBandwidthLimitRuleTests(
        ProjectMemberQosBandwidthLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosBandwidthLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy_bandwidth_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_bandwidth_limit_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_bandwidth_limit_rule',
                self.target)

    def test_create_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_bandwidth_limit_rule',
            self.target)

    def test_update_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_bandwidth_limit_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_bandwidth_limit_rule',
            self.target)

    def test_delete_policy_bandwidth_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_bandwidth_limit_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_bandwidth_limit_rule',
            self.target)


class SystemAdminQosPacketRateLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_policy_packet_rate_limit_rule',
            self.alt_target)

    def test_create_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_packet_rate_limit_rule',
            self.alt_target)

    def test_update_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_packet_rate_limit_rule',
            self.alt_target)

    def test_delete_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_packet_rate_limit_rule',
            self.alt_target)


class AdminQosPacketRateLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy_packet_rate_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_packet_rate_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_packet_rate_limit_rule',
                               self.alt_target))

    def test_create_policy_packet_rate_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_packet_rate_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_packet_rate_limit_rule',
                           self.alt_target))

    def test_update_policy_packet_rate_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_packet_rate_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_packet_rate_limit_rule',
                           self.alt_target))

    def test_delete_policy_packet_rate_limit_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_packet_rate_limit_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_packet_rate_limit_rule',
                           self.alt_target))


class ProjectManagerQosPacketRateLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy_packet_rate_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_packet_rate_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_packet_rate_limit_rule',
                self.alt_target)

    def test_create_policy_packet_rate_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'create_policy_packet_rate_limit_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'create_policy_packet_rate_limit_rule',
                self.alt_target)

    def test_update_policy_packet_rate_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'update_policy_packet_rate_limit_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_policy_packet_rate_limit_rule',
                self.alt_target)

    def test_delete_policy_packet_rate_limit_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_policy_packet_rate_limit_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_policy_packet_rate_limit_rule',
                self.alt_target)


class ProjectMemberQosPacketRateLimitRuleTests(
        ProjectManagerQosPacketRateLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_create_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_packet_rate_limit_rule',
            self.alt_target)

    def test_update_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_packet_rate_limit_rule',
            self.alt_target)

    def test_delete_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_packet_rate_limit_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_packet_rate_limit_rule',
            self.alt_target)


class ProjectReaderQosPacketRateLimitRuleTests(
        ProjectMemberQosPacketRateLimitRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosPacketRateLimitRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_policy_packet_rate_limit_rule',
            self.target)

    def test_create_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_packet_rate_limit_rule',
            self.target)

    def test_update_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_packet_rate_limit_rule',
            self.target)

    def test_delete_policy_packet_rate_limit_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_packet_rate_limit_rule',
            self.target)


class SystemAdminQosDSCPMarkingRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_dscp_marking_rule',
                self.target)

            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_dscp_marking_rule',
                self.target)

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_dscp_marking_rule',
                self.alt_target)

            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_dscp_marking_rule',
                self.alt_target)

    def test_create_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_dscp_marking_rule',
            self.alt_target)

    def test_update_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_dscp_marking_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_dscp_marking_rule',
            self.alt_target)

    def test_delete_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_dscp_marking_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_dscp_marking_rule',
            self.alt_target)


class SystemMemberQosDSCPMarkingRuleTests(SystemAdminQosDSCPMarkingRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosDSCPMarkingRuleTests(SystemMemberQosDSCPMarkingRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosDSCPMarkingRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_dscp_marking_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_dscp_marking_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_dscp_marking_rule',
                               self.alt_target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_dscp_marking_rule',
                               self.alt_target))

    def test_create_policy_dscp_marking_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_dscp_marking_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_dscp_marking_rule',
                           self.alt_target))

    def test_update_policy_dscp_marking_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_dscp_marking_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_dscp_marking_rule',
                           self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_dscp_marking_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_dscp_marking_rule',
                           self.alt_target))

    def test_delete_policy_dscp_marking_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_dscp_marking_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_dscp_marking_rule',
                           self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_dscp_marking_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_dscp_marking_rule',
                           self.alt_target))


class ProjectManagerQosDSCPMarkingRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_dscp_marking_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_dscp_marking_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_dscp_marking_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_dscp_marking_rule',
                self.alt_target)

    def test_create_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'create_policy_dscp_marking_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'create_policy_dscp_marking_rule',
                self.alt_target)

    def test_update_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'update_policy_dscp_marking_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'update_alias_dscp_marking_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_policy_dscp_marking_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_alias_dscp_marking_rule',
                self.alt_target)

    def test_delete_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_policy_dscp_marking_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_alias_dscp_marking_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_policy_dscp_marking_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_alias_dscp_marking_rule',
                self.alt_target)


class ProjectMemberQosDSCPMarkingRuleTests(
        ProjectManagerQosDSCPMarkingRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_create_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_dscp_marking_rule',
            self.alt_target)

    def test_update_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_dscp_marking_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_dscp_marking_rule',
            self.alt_target)

    def test_delete_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_dscp_marking_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_dscp_marking_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_dscp_marking_rule',
            self.alt_target)


class ProjectReaderQosDSCPMarkingRuleTests(
        ProjectMemberQosDSCPMarkingRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosDSCPMarkingRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy_dscp_marking_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_dscp_marking_rule',
                self.target)

            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_dscp_marking_rule',
                self.target)

    def test_create_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_dscp_marking_rule',
            self.target)

    def test_update_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_dscp_marking_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_dscp_marking_rule',
            self.target)

    def test_delete_policy_dscp_marking_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_dscp_marking_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_dscp_marking_rule',
            self.target)


class SystemAdminQosMinimumBandwidthRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_minimum_bandwidth_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_minimum_bandwidth_rule',
                self.target)

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_minimum_bandwidth_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_minimum_bandwidth_rule',
                self.alt_target)

    def test_create_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_minimum_bandwidth_rule',
            self.alt_target)

    def test_update_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_minimum_bandwidth_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_alias_minimum_bandwidth_rule',
            self.alt_target)

    def test_delete_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_minimum_bandwidth_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_minimum_bandwidth_rule',
            self.alt_target)


class SystemMemberQosMinimumBandwidthRuleTests(
        SystemAdminQosMinimumBandwidthRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosMinimumBandwidthRuleTests(
        SystemMemberQosMinimumBandwidthRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosMinimumBandwidthRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_policy_minimum_bandwidth_rule',
                    self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_alias_minimum_bandwidth_rule',
                    self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_policy_minimum_bandwidth_rule',
                    self.alt_target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_alias_minimum_bandwidth_rule',
                    self.alt_target))

    def test_create_policy_minimum_bandwidth_rule(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'create_policy_minimum_bandwidth_rule',
                self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'create_policy_minimum_bandwidth_rule',
                self.alt_target))

    def test_update_policy_minimum_bandwidth_rule(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'update_policy_minimum_bandwidth_rule',
                self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'update_policy_minimum_bandwidth_rule',
                self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(
                self.context, 'update_alias_minimum_bandwidth_rule',
                self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'update_alias_minimum_bandwidth_rule',
                self.alt_target))

    def test_delete_policy_minimum_bandwidth_rule(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'delete_policy_minimum_bandwidth_rule',
                self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'delete_policy_minimum_bandwidth_rule',
                self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(
                self.context, 'delete_alias_minimum_bandwidth_rule',
                self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'delete_alias_minimum_bandwidth_rule',
                self.alt_target))


class ProjectManagerQosMinimumBandwidthRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_policy_minimum_bandwidth_rule',
                    self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(
                    self.context, 'get_alias_minimum_bandwidth_rule',
                    self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_minimum_bandwidth_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_minimum_bandwidth_rule',
                self.alt_target)

    def test_create_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'create_policy_minimum_bandwidth_rule',
                    self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'create_policy_minimum_bandwidth_rule',
                self.alt_target)

    def test_update_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'update_policy_minimum_bandwidth_rule',
                    self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(
                    self.context, 'update_alias_minimum_bandwidth_rule',
                    self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_policy_minimum_bandwidth_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_alias_minimum_bandwidth_rule',
                self.alt_target)

    def test_delete_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(
                    self.context, 'delete_policy_minimum_bandwidth_rule',
                    self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(
                    self.context, 'delete_alias_minimum_bandwidth_rule',
                    self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_policy_minimum_bandwidth_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_alias_minimum_bandwidth_rule',
                self.alt_target)


class ProjectMemberQosMinimumBandwidthRuleTests(
        ProjectManagerQosMinimumBandwidthRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_create_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_bandwidth_rule',
            self.alt_target)

    def test_update_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_bandwidth_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_minimum_bandwidth_rule',
            self.alt_target)

    def test_delete_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_bandwidth_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_bandwidth_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_bandwidth_rule',
            self.alt_target)


class ProjectReaderQosMinimumBandwidthRuleTests(
        ProjectMemberQosMinimumBandwidthRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosMinimumBandwidthRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy_minimum_bandwidth_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_minimum_bandwidth_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_minimum_bandwidth_rule',
                self.target)

    def test_create_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_bandwidth_rule',
            self.target)

    def test_update_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_bandwidth_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_minimum_bandwidth_rule',
            self.target)

    def test_delete_policy_minimum_bandwidth_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_bandwidth_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_bandwidth_rule',
            self.target)


class SystemAdminQosMinimumPacketRateRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_get_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_minimum_packet_rate_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_minimum_packet_rate_rule',
                self.target)

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_policy_minimum_packet_rate_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.InvalidScope,
                policy.enforce,
                self.context, 'get_alias_minimum_packet_rate_rule',
                self.alt_target)

    def test_create_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_policy_minimum_packet_rate_rule',
            self.alt_target)

    def test_update_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_policy_minimum_packet_rate_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_alias_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_alias_minimum_packet_rate_rule',
            self.alt_target)

    def test_delete_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_policy_minimum_packet_rate_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_alias_minimum_packet_rate_rule',
            self.alt_target)


class SystemMemberQosMinimumPacketRateRuleTests(
        SystemAdminQosMinimumPacketRateRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderQosMinimumPacketRateRuleTests(
        SystemMemberQosMinimumPacketRateRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminQosMinimumPacketRateRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_get_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_minimum_packet_rate_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_minimum_packet_rate_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_minimum_packet_rate_rule',
                               self.alt_target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_minimum_packet_rate_rule',
                               self.alt_target))

    def test_create_policy_minimum_packet_rate_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_minimum_packet_rate_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'create_policy_minimum_packet_rate_rule',
                           self.alt_target))

    def test_update_policy_minimum_packet_rate_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_minimum_packet_rate_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_policy_minimum_packet_rate_rule',
                           self.alt_target))

        # And the same for aliases
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_minimum_packet_rate_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'update_alias_minimum_packet_rate_rule',
                           self.alt_target))

    def test_delete_policy_minimum_packet_rate_rule(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_minimum_packet_rate_rule',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'delete_policy_minimum_packet_rate_rule',
                           self.alt_target))


class ProjectManagerQosMinimumPacketRateRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_get_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'get_policy_minimum_packet_rate_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'get_alias_minimum_packet_rate_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_minimum_packet_rate_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_minimum_packet_rate_rule',
                self.alt_target)

    def test_create_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'create_policy_minimum_packet_rate_rule',
                               self.target))
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'create_policy_minimum_packet_rate_rule',
                self.alt_target)

    def test_update_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'update_policy_minimum_packet_rate_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'update_alias_minimum_packet_rate_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_policy_minimum_packet_rate_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'update_alias_minimum_packet_rate_rule',
                self.alt_target)

    def test_delete_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_policy_minimum_packet_rate_rule',
                               self.target))
            # And the same for aliases
            self.assertTrue(
                policy.enforce(self.context,
                               'delete_alias_minimum_packet_rate_rule',
                               self.target))

        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.alt_qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_policy_minimum_packet_rate_rule',
                self.alt_target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'delete_alias_minimum_packet_rate_rule',
                self.alt_target)


class ProjectMemberQosMinimumPacketRateRuleTests(
        ProjectManagerQosMinimumPacketRateRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx

    def test_create_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_packet_rate_rule',
            self.alt_target)

    def test_update_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_packet_rate_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_alias_minimum_packet_rate_rule',
            self.alt_target)

    def test_delete_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_packet_rate_rule',
            self.alt_target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_packet_rate_rule',
            self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_packet_rate_rule',
            self.alt_target)


class ProjectReaderQosMinimumPacketRateRuleTests(
        ProjectMemberQosMinimumPacketRateRuleTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx


class ServiceRoleQosMinimumPacketRateRuleTests(QosRulesAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_get_policy_minimum_packet_rate_rule(self):
        with mock.patch.object(self.plugin_mock, "get_policy",
                               return_value=self.qos_policy):
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_policy_minimum_packet_rate_rule',
                self.target)
            # And the same for aliases
            self.assertRaises(
                base_policy.PolicyNotAuthorized,
                policy.enforce,
                self.context, 'get_alias_minimum_packet_rate_rule',
                self.target)

    def test_create_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_policy_minimum_packet_rate_rule',
            self.target)

    def test_update_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_policy_minimum_packet_rate_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_alias_minimum_packet_rate_rule',
            self.target)

    def test_delete_policy_minimum_packet_rate_rule(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_policy_minimum_packet_rate_rule',
            self.target)

        # And the same for aliases
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_alias_minimum_packet_rate_rule',
            self.target)



# Copyright (c) 2021 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_policy import policy as base_policy

from neutron import policy
from neutron.tests.unit.conf.policies import test_base as base


class SubnetpoolAPITestCase(base.PolicyBaseTestCase):

    def setUp(self):
        super().setUp()
        self.target = {'project_id': self.project_id}
        self.alt_target = {'project_id': self.alt_project_id}


class SystemAdminTests(SubnetpoolAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.system_admin_ctx

    def test_create_subnetpool(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool', self.alt_target)

    def test_create_subnetpool_shared(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:shared', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:shared', self.alt_target)

    def test_create_subnetpool_default(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:is_default', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:is_default', self.alt_target)

    def test_create_subnetpool_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.alt_target)

    def test_get_subnetpool(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_subnetpool', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_subnetpool', self.alt_target)

    def test_get_subnetpool_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'get_subnetpool:tags', self.alt_target)

    def test_update_subnetpool(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool', self.alt_target)

    def test_update_subnetpool_default(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool:is_default', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool:is_default', self.alt_target)

    def test_update_subnetpool_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.alt_target)

    def test_delete_subnetpool(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_subnetpool', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_subnetpool', self.alt_target)

    def test_delete_subnetpool_tags(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.alt_target)

    def test_onboard_network_subnets(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.alt_target)

    def test_add_prefixes(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'add_prefixes', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'add_prefixes', self.alt_target)

    def test_remove_prefixes(self):
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'remove_prefixes', self.target)
        self.assertRaises(
            base_policy.InvalidScope,
            policy.enforce,
            self.context, 'remove_prefixes', self.alt_target)


class SystemMemberTests(SystemAdminTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_member_ctx


class SystemReaderTests(SystemMemberTests):

    def setUp(self):
        super().setUp()
        self.context = self.system_reader_ctx


class AdminTests(SubnetpoolAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.project_admin_ctx

    def test_create_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_subnetpool', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'create_subnetpool', self.alt_target))

    def test_create_subnetpool_shared(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'create_subnetpool:shared', self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'create_subnetpool:shared', self.alt_target))

    def test_create_subnetpool_default(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'create_subnetpool:default', self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'create_subnetpool:default', self.alt_target))

    def test_create_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_subnetpool:tags',
                           self.target))
        self.assertTrue(policy.enforce(self.context, 'create_subnetpool:tags',
                                       self.alt_target))

    def test_get_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool', self.alt_target))

    def test_get_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool:tags', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool:tags',
                           self.alt_target))

    def test_update_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool', self.alt_target))

    def test_update_subnetpool_default(self):
        self.assertTrue(
            policy.enforce(
                self.context, 'update_subnetpool:default', self.target))
        self.assertTrue(
            policy.enforce(
                self.context, 'update_subnetpool:default', self.alt_target))

    def test_update_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool:tags',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool:tags',
                           self.alt_target))

    def test_delete_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool', self.alt_target))

    def test_delete_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool:tags',
                           self.target))
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool:tags',
                           self.alt_target))

    def test_onboard_network_subnets(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'onboard_network_subnets', self.target))
        self.assertTrue(
            policy.enforce(self.context,
                           'onboard_network_subnets', self.alt_target))

    def test_add_prefixes(self):
        self.assertTrue(
            policy.enforce(self.context, 'add_prefixes', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'add_prefixes', self.alt_target))

    def test_remove_prefixes(self):
        self.assertTrue(
            policy.enforce(self.context, 'remove_prefixes', self.target))
        self.assertTrue(
            policy.enforce(self.context, 'remove_prefixes', self.alt_target))


class ProjectManagerTests(AdminTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_manager_ctx

    def test_create_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_subnetpool', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool', self.alt_target)

    def test_create_subnetpool_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:shared', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:shared', self.alt_target)

    def test_create_subnetpool_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:is_default', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:is_default', self.alt_target)

    def test_create_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'create_subnetpool:tags',
                           self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.alt_target)

    def test_get_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_subnetpool', self.alt_target)

    def test_get_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'get_subnetpool:tags', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_subnetpool:tags', self.alt_target)

    def test_update_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool', self.alt_target)

    def test_update_subnetpool_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:is_default', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:is_default', self.alt_target)

    def test_update_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'update_subnetpool:tags',
                           self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.alt_target)

    def test_delete_subnetpool(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool', self.alt_target)

    def test_delete_subnetpool_tags(self):
        self.assertTrue(
            policy.enforce(self.context, 'delete_subnetpool:tags',
                           self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.alt_target)

    def test_onboard_network_subnets(self):
        self.assertTrue(
            policy.enforce(self.context,
                           'onboard_network_subnets', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.alt_target)

    def test_add_prefixes(self):
        self.assertTrue(
            policy.enforce(self.context, 'add_prefixes', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'add_prefixes', self.alt_target)

    def test_remove_prefixes(self):
        self.assertTrue(
            policy.enforce(self.context, 'remove_prefixes', self.target))
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'remove_prefixes', self.alt_target)


class ProjectMemberTests(ProjectManagerTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_member_ctx


class ProjectReaderTests(ProjectMemberTests):

    def setUp(self):
        super().setUp()
        self.context = self.project_reader_ctx

    def test_create_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool', self.alt_target)

    def test_create_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.alt_target)

    def test_update_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool', self.alt_target)

    def test_update_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.alt_target)

    def test_delete_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool', self.alt_target)

    def test_delete_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.alt_target)

    def test_onboard_network_subnets(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.alt_target)

    def test_add_prefixes(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'add_prefixes', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'add_prefixes', self.alt_target)

    def test_remove_prefixes(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'remove_prefixes', self.target)
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'remove_prefixes', self.alt_target)


class ServiceRoleTests(SubnetpoolAPITestCase):

    def setUp(self):
        super().setUp()
        self.context = self.service_ctx

    def test_create_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool', self.target)

    def test_create_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:tags', self.target)

    def test_create_subnetpool_shared(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:shared', self.target)

    def test_create_subnetpool_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'create_subnetpool:is_default', self.target)

    def test_get_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_subnetpool', self.target)

    def test_get_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'get_subnetpool:tags', self.target)

    def test_update_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool', self.target)

    def test_update_subnetpool_default(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:is_default', self.target)

    def test_update_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'update_subnetpool:tags', self.target)

    def test_delete_subnetpool(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool', self.target)

    def test_delete_subnetpool_tags(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'delete_subnetpool:tags', self.target)

    def test_onboard_network_subnets(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'onboard_network_subnets', self.target)

    def test_add_prefixes(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'add_prefixes', self.target)

    def test_remove_prefixes(self):
        self.assertRaises(
            base_policy.PolicyNotAuthorized,
            policy.enforce,
            self.context, 'remove_prefixes', self.target)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron_lib.api.definitions import port_mac_address_regenerate
from neutron_lib.api import extensions as api_extensions


class Port_mac_address_regenerate(api_extensions.APIExtensionDescriptor):
    """Extension to support port MAC address regeneration"""

    api_definition = port_mac_address_regenerate



# Copyright 2024 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.callbacks import events
from neutron_lib.callbacks import priority_group
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources

from neutron.common.ovn import utils
from neutron.db import l3_attrs_db


@registry.has_registry_receivers
class OVN_L3_HA_db_mixin(l3_attrs_db.ExtraAttributesMixin):
    """Mixin class to add high availability capability to OVN routers."""

    @registry.receives(resources.ROUTER, [events.PRECOMMIT_CREATE],
                       priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
    def _precommit_router_create(self, resource, event, trigger, payload):
        """Event handler to set ha flag creation."""
        # NOTE(ralonsoh): OVN L3 router HA flag is mandatory and True always,
        # enforced by ``OvnDriver.ha_support`` set to ``MANDATORY``. This flag
        # cannot be updated.
        router = payload.latest_state
        if not utils.is_ovn_provider_router(router):
            return
        router_db = payload.metadata['router_db']
        self.set_extra_attr_value(router_db, 'ha', True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import portbindings
from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib import constants
from neutron_lib import context as n_context
from neutron_lib.db import api as db_api
from neutron_lib import exceptions as n_exc
from neutron_lib.services.trunk import constants as trunk_consts
from oslo_config import cfg
from oslo_log import log

from neutron.common.ovn import constants as ovn_const
from neutron.db import db_base_plugin_common
from neutron.db import ovn_revision_numbers_db as db_rev
from neutron.objects import ports as port_obj
from neutron.objects import trunk as trunk_objects
from neutron.services.trunk.drivers import base as trunk_base
from neutron.services.trunk import exceptions as trunk_exc


SUPPORTED_INTERFACES = (
    portbindings.VIF_TYPE_OVS,
    portbindings.VIF_TYPE_VHOST_USER,
)

SUPPORTED_SEGMENTATION_TYPES = (
    trunk_consts.SEGMENTATION_TYPE_VLAN,
)

LOG = log.getLogger(__name__)


class OVNTrunkHandler:
    def __init__(self, plugin_driver):
        self.plugin_driver = plugin_driver

    def _set_sub_ports(self, parent_port, subports):
        txn = self.plugin_driver.nb_ovn.transaction
        context = n_context.get_admin_context()
        db_parent_port = port_obj.Port.get_object(context, id=parent_port)
        parent_port_status = db_parent_port.status
        for subport in subports:
            with db_api.CONTEXT_WRITER.using(context), (
                    txn(check_error=True)) as ovn_txn:
                port = self._set_binding_profile(context, subport, parent_port,
                                                 parent_port_status, ovn_txn)
            db_rev.bump_revision(context, port, ovn_const.TYPE_PORTS)

    def _unset_sub_ports(self, subports):
        txn = self.plugin_driver.nb_ovn.transaction
        context = n_context.get_admin_context()
        for subport in subports:
            with db_api.CONTEXT_WRITER.using(context), (
                    txn(check_error=True)) as ovn_txn:
                port = self._unset_binding_profile(context, subport, ovn_txn)
            db_rev.bump_revision(context, port, ovn_const.TYPE_PORTS)

    @db_base_plugin_common.convert_result_to_dict
    def _set_binding_profile(self, context, subport, parent_port,
                             parent_port_status, ovn_txn):
        LOG.debug("Setting parent %s for subport %s",
                  parent_port, subport.port_id)
        db_port = port_obj.Port.get_object(context, id=subport.port_id)
        if not db_port:
            LOG.debug("Port not found while trying to set "
                      "binding_profile: %s",
                      subport.port_id)
            return
        check_rev_cmd = self.plugin_driver.nb_ovn.check_revision_number(
            db_port.id, db_port, ovn_const.TYPE_PORTS)
        ovn_txn.add(check_rev_cmd)
        try:
            # NOTE(flaviof): We expect binding's host to be set. Otherwise,
            # sub-port will not transition from DOWN to ACTIVE.
            db_port.device_owner = trunk_consts.TRUNK_SUBPORT_OWNER
            # NOTE(ksambor):  When sub-port was created and event was process
            # without binding profile this port will end forever in DOWN
            # status so we need to switch it here to the parent port status
            db_port.status = parent_port_status
            for binding in db_port.bindings:
                binding.profile['parent_name'] = parent_port
                binding.profile['tag'] = subport.segmentation_id
                # host + port_id is primary key
                port_obj.PortBinding.update_object(
                    context,
                    {'profile': binding.profile,
                     'vif_type': portbindings.VIF_TYPE_OVS},
                    port_id=subport.port_id,
                    host=binding.host)
            db_port.update()
        except n_exc.ObjectNotFound:
            LOG.debug("Port not found while trying to set "
                      "binding_profile: %s", subport.port_id)
            return
        ext_ids = {ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY: db_port.device_owner}
        ovn_txn.add(self.plugin_driver.nb_ovn.set_lswitch_port(
            lport_name=subport.port_id,
            parent_name=parent_port,
            tag=subport.segmentation_id,
            external_ids_update=ext_ids,
        ))
        LOG.debug("Done setting parent %s for subport %s",
                  parent_port, subport.port_id)
        return db_port

    @db_base_plugin_common.convert_result_to_dict
    def _unset_binding_profile(self, context, subport, ovn_txn):
        LOG.debug("Unsetting parent for subport %s", subport.port_id)
        db_port = port_obj.Port.get_object(context, id=subport.port_id)
        if not db_port:
            LOG.debug("Port not found while trying to unset "
                      "binding_profile: %s",
                      subport.port_id)
            return
        check_rev_cmd = self.plugin_driver.nb_ovn.check_revision_number(
            db_port.id, db_port, ovn_const.TYPE_PORTS)
        ovn_txn.add(check_rev_cmd)
        try:
            db_port.device_owner = ''
            for binding in db_port.bindings:
                binding.profile.pop('tag', None)
                binding.profile.pop('parent_name', None)
                # host + port_id is primary key
                port_obj.PortBinding.update_object(
                    context,
                    {'profile': binding.profile,
                     'vif_type': portbindings.VIF_TYPE_UNBOUND},
                    port_id=subport.port_id,
                    host=binding.host)
                port_obj.PortBindingLevel.delete_objects(
                    context, port_id=subport.port_id, host=binding.host)
            db_port.update()
        except n_exc.ObjectNotFound:
            LOG.debug("Port not found while trying to unset "
                      "binding_profile: %s", subport.port_id)
            return
        ext_ids = {ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY: db_port.device_owner}
        ovn_txn.add(self.plugin_driver.nb_ovn.set_lswitch_port(
            lport_name=subport.port_id,
            parent_name=[],
            up=False,
            tag=[],
            external_ids_update=ext_ids,
        ))
        LOG.debug("Done unsetting parent for subport %s", subport.port_id)
        return db_port

    def trunk_created(self, resource, event, trunk_plugin, payload):
        trunk = payload.states[0]
        # Check if parent port is handled by OVN.
        if not self.plugin_driver.nb_ovn.lookup('Logical_Switch_Port',
                                                trunk.port_id, default=None):
            return
        if trunk.sub_ports:
            self._set_sub_ports(trunk.port_id, trunk.sub_ports)
        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)

    def trunk_deleted(self, resource, event, trunk_plugin, payload):
        trunk = payload.states[0]
        if trunk.sub_ports:
            self._unset_sub_ports(trunk.sub_ports)

    def subports_added(self, resource, event, trunk_plugin, payload):
        trunk = payload.states[0]
        subports = payload.metadata['subports']
        # Check if parent port is handled by OVN.
        if not self.plugin_driver.nb_ovn.lookup('Logical_Switch_Port',
                                                trunk.port_id, default=None):
            return
        if subports:
            self._set_sub_ports(trunk.port_id, subports)
        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)

    def subports_deleted(self, resource, event, trunk_plugin, payload):
        trunk = payload.states[0]
        subports = payload.metadata['subports']
        # Check if parent port is handled by OVN.
        if not self.plugin_driver.nb_ovn.lookup('Logical_Switch_Port',
                                                trunk.port_id, default=None):
            return
        if subports:
            self._unset_sub_ports(subports)
        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)

    def port_updated(self, resource, event, trunk_plugin, payload):
        '''Propagate trunk parent port ACTIVE to trunk ACTIVE

        During a live migration with a trunk the only way we found to update
        the trunk to ACTIVE is to do this when the trunk's parent port gets
        updated to ACTIVE. This is clearly suboptimal, because the trunk's
        ACTIVE status should mean that all of its ports (parent and sub) are
        active. But in ml2/ovn the parent port's binding is not cascaded to the
        subports. Actually the subports' binding:host is left empty. This way
        here we don't know anything about the subports' state changes during a
        live migration. If we don't want to leave the trunk in DOWN this is
        what we have.

        Please note that this affects trunk create as well. Because of this we
        move ml2/ovn trunks to ACTIVE way early. But at least here we don't
        affect other mechanism drivers and their corresponding trunk drivers.

        See also:
        https://bugs.launchpad.net/neutron/+bug/1988549
        https://review.opendev.org/c/openstack/neutron/+/853779
        https://bugs.launchpad.net/neutron/+bug/2095152
        '''
        updated_port = payload.latest_state
        trunk_details = updated_port.get('trunk_details')
        # If no trunk_details, the port is not the parent of a trunk.
        if not trunk_details:
            return

        original_port = payload.states[0]
        orig_status = original_port.get('status')
        new_status = updated_port.get('status')
        context = payload.context
        trunk_id = trunk_details['trunk_id']
        if (new_status == constants.PORT_STATUS_ACTIVE and
                new_status != orig_status):
            trunk = trunk_objects.Trunk.get_object(context, id=trunk_id)
            if trunk is None:
                raise trunk_exc.TrunkNotFound(trunk_id=trunk_id)
            trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)


class OVNTrunkDriver(trunk_base.DriverBase):
    @property
    def is_loaded(self):
        try:
            return (ovn_const.OVN_ML2_MECH_DRIVER_NAME in
                    cfg.CONF.ml2.mechanism_drivers)
        except cfg.NoSuchOptError:
            return False

    @registry.receives(resources.TRUNK_PLUGIN, [events.AFTER_INIT])
    def register(self, resource, event, trigger, payload=None):
        super().register(
            resource, event, trigger, payload=payload)
        self._handler = OVNTrunkHandler(self.plugin_driver)

        registry.subscribe(
            self._handler.trunk_created, resources.TRUNK, events.AFTER_CREATE)
        registry.subscribe(
            self._handler.trunk_deleted, resources.TRUNK, events.AFTER_DELETE)

        registry.subscribe(
            self._handler.subports_added,
            resources.SUBPORTS,
            events.AFTER_CREATE)
        registry.subscribe(
            self._handler.subports_deleted,
            resources.SUBPORTS,
            events.AFTER_DELETE)

        registry.subscribe(
            self._handler.port_updated,
            resources.PORT,
            events.AFTER_UPDATE)

    @classmethod
    def create(cls, plugin_driver):
        cls.plugin_driver = plugin_driver
        return cls(ovn_const.OVN_ML2_MECH_DRIVER_NAME,
                   SUPPORTED_INTERFACES,
                   SUPPORTED_SEGMENTATION_TYPES,
                   None,
                   can_trunk_bound_port=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from unittest import mock

from neutron_lib.callbacks import events
from neutron_lib import constants as const
from oslo_utils import uuidutils

from neutron.db.models import l3
from neutron.db.models import l3_attrs
from neutron.objects import router as l3_obj
from neutron.services.ovn_l3.service_providers import user_defined
from neutron.tests.unit import testlib_api


DB_PLUGIN_KLASS = 'neutron.db.db_base_plugin_v2.NeutronDbPluginV2'


class TestUserDefined(testlib_api.SqlTestCase):

    def setUp(self):
        super().setUp()
        self.setup_coreplugin(DB_PLUGIN_KLASS)
        self.fake_l3 = mock.MagicMock()
        self.fake_l3._make_router_interface_info = mock.MagicMock(
            return_value='router_interface_info')
        self.provider = user_defined.UserDefined(self.fake_l3)
        self.context = 'fake-context'
        self.router = l3.Router(id='fake-uuid',
                                flavor_id='fake-uuid')
        self.router['extra_attributes'] = l3_attrs.RouterExtraAttributes()
        self.fake_l3.get_router = mock.MagicMock(return_value=self.router)
        self.fip = {'router_id': 'fake-uuid'}
        mock_flavor_plugin = mock.MagicMock()
        mock_flavor_plugin.get_flavor = mock.MagicMock(
            return_value={'id': 'fake-uuid'})
        mock_flavor_plugin.get_flavor_next_provider = mock.MagicMock(
            return_value=[{'driver': self.provider._user_defined_provider}])
        self.provider._flavor_plugin_ref = mock_flavor_plugin

    def test__is_user_defined_provider(self):
        # test the positive case
        self.assertTrue(self.provider._is_user_defined_provider(
            self.context, self.router))

        # test the negative case
        self.provider._flavor_plugin_ref.get_flavor_next_provider = (
            mock.MagicMock(return_value=[{'driver': None}]))
        self.assertFalse(self.provider._is_user_defined_provider(
            self.context, self.router))

        # test flavor_id request not specified
        self.router.flavor_id = None
        self.assertFalse(self.provider._is_user_defined_provider(
            self.context, self.router))
        self.router.flavor_id = const.ATTR_NOT_SPECIFIED
        self.assertFalse(self.provider._is_user_defined_provider(
            self.context, self.router))

    def test_router_processing(self):
        with mock.patch.object(user_defined.LOG, 'debug') as log:
            payload = events.DBEventPayload(
                self.context,
                states=(self.router, self.router),
                resource_id=self.router['id'],
                metadata={'subnet_ids': ['subnet-id']})
            fl_plg = self.provider._flavor_plugin_ref
            methods = [self.provider._process_router_add_association,
                       self.provider._process_router_create,
                       self.provider._process_router_update,
                       self.provider._process_router_delete,
                       self.provider._process_remove_router_interface]
            for method in methods:
                method('resource', 'event', self, payload)
                fl_plg.get_flavor.assert_called_once()
                fl_plg.get_flavor_next_provider.assert_called_once()
                log.assert_called_once()
                fl_plg.get_flavor.reset_mock()
                fl_plg.get_flavor_next_provider.reset_mock()
                log.reset_mock()

    def test_add_router_interface(self):
        with mock.patch.object(user_defined.LOG, 'debug') as log:
            payload = events.DBEventPayload(
                self.context,
                states=(self.router, self.router),
                resource_id=self.router['id'],
                metadata={'subnet_ids': ['subnet-id'],
                          'port': {'project_id': 'project-id',
                                   'id': 'id',
                                   'network_id': 'network-id'},
                          'subnets': [{'id': 'id'}]})
            fl_plg = self.provider._flavor_plugin_ref
            l3_plg = self.fake_l3
            self.provider._process_add_router_interface('resource',
                                                        'event',
                                                        self,
                                                        payload)
            l3_plg._make_router_interface_info.assert_called_once()
            fl_plg.get_flavor.assert_called_once()
            fl_plg.get_flavor_next_provider.assert_called_once()
            log.assert_called_once()

    def test_floatingip_processing(self):
        # Test all the methods related to FIP processing
        with mock.patch.object(user_defined.LOG, 'debug') as log:
            payload = events.DBEventPayload(
                self.context,
                states=(self.fip, self.fip))
            fl_plg = self.provider._flavor_plugin_ref
            l3_plg = self.fake_l3
            methods = [self.provider._process_floatingip_create,
                       self.provider._process_floatingip_update,
                       self.provider._process_floatingip_delete,
                       self.provider._process_floatingip_status_update]
            for method in methods:
                method('resource', 'event', self, payload)
                l3_plg.get_router.assert_called_once()
                fl_plg.get_flavor.assert_called_once()
                fl_plg.get_flavor_next_provider.assert_called_once()
                log.assert_called_once()
                l3_plg.get_router.reset_mock()
                fl_plg.get_flavor.reset_mock()
                fl_plg.get_flavor_next_provider.reset_mock()
                log.reset_mock()

    def test__is_ha(self):
        # test the positive case
        router_req = {'id': 'fake-uuid',
                      'flavor_id': 'fake-uuid',
                      'ha': True}
        self.assertTrue(self.provider._is_ha(router_req))

        # test the negative case
        router_req['ha'] = False
        self.assertFalse(self.provider._is_ha(router_req))

    @mock.patch('neutron.db.l3_attrs_db.get_attr_info')
    def test__process_precommit_router_create(self, gai):
        gai.return_value = {'ha': {'default': False}}
        router_req = {'id': 'fake-uuid',
                      'flavor_id': 'fake-uuid',
                      'ha': True}
        payload = events.DBEventPayload(
            self.context,
            resource_id=self.router['id'],
            states=(router_req,),
            metadata={'router_db': self.router})
        self.assertFalse(self.router['extra_attributes'].ha)
        self.provider._process_precommit_router_create('resource', 'event',
                                                       self, payload)
        self.assertTrue(self.router['extra_attributes'].ha)


class TestUserDefinedNoLsp(testlib_api.SqlTestCase):

    def setUp(self):
        super().setUp()
        self.setup_coreplugin(DB_PLUGIN_KLASS)
        self.fake_l3 = mock.MagicMock()
        self.fake_l3._make_router_interface_info = mock.MagicMock(
            return_value='router_interface_info')
        self.provider = user_defined.UserDefinedNoLsp(self.fake_l3)
        self.context = mock.MagicMock()
        self.router = l3.Router(id='fake-uuid',
                                flavor_id='fake-uuid')
        mock_flavor_plugin = mock.MagicMock()
        mock_flavor_plugin.get_flavor = mock.MagicMock(
            return_value={'id': 'fake-uuid'})
        mock_flavor_plugin.get_flavor_next_provider = mock.MagicMock(
            return_value=[{'driver': self.provider._user_defined_provider}])
        self.provider._flavor_plugin_ref = mock_flavor_plugin

    @mock.patch.object(user_defined.LOG, 'debug')
    def test__add_router_interface(self, log_mock):
        payload = events.DBEventPayload(
            self.context,
            states=(self.router, self.router),
            resource_id=self.router['id'],
            metadata={'subnet_ids': ['subnet-id'],
                      'port': {'project_id': 'project-id',
                               'id': 'id',
                               'network_id': 'network-id'},
                      'subnets': [{'id': 'id'}]})
        fl_plg = self.provider._flavor_plugin_ref
        l3_plg = self.fake_l3
        self.provider._process_add_router_interface('resource',
                                                    'event',
                                                    self,
                                                    payload)
        l3_plg._make_router_interface_info.assert_called_once()
        fl_plg.get_flavor.assert_called_once()
        fl_plg.get_flavor_next_provider.assert_called_once()
        l3_plg._ovn_client.delete_port.assert_called_once()
        log_mock.assert_called_once()

    @mock.patch.object(l3_obj.RouterPort, 'get_objects')
    @mock.patch.object(l3_obj.Router, 'get_object')
    @mock.patch.object(user_defined.LOG, 'debug')
    def _test__process_before_remove_router_interface(self, port_exists,
                                                      log_mock, grouter_mock,
                                                      get_objects_mock):
        payload = events.DBEventPayload(
            self.context,
            resource_id=self.router['id'],
            metadata={'subnet_id': 'subnet-id'})
        grouter_mock.return_value = {'id': 'fake-uuid',
                                     'flavor_id': 'fake-uuid'}
        nbdb_idl_mock = mock.MagicMock()
        nbdb_idl_mock.lookup = mock.MagicMock()
        if port_exists:
            nbdb_idl_mock.lookup.return_value = 'a-port'
        else:
            nbdb_idl_mock.lookup.return_value = None
        self.fake_l3._ovn_client = mock.MagicMock()
        self.fake_l3._ovn_client._nb_idl = nbdb_idl_mock

        port_id = uuidutils.generate_uuid()
        other_port_id = uuidutils.generate_uuid()
        rp = l3_obj.RouterPort(
                self.context,
                port_id=port_id,
                router_id=uuidutils.generate_uuid(),
                port_type=const.DEVICE_OWNER_ROUTER_INTF)
        rp.create()
        other_rp = l3_obj.RouterPort(
                self.context,
                port_id=other_port_id,
                router_id=uuidutils.generate_uuid(),
                port_type=const.DEVICE_OWNER_ROUTER_INTF)
        other_rp.create()
        get_objects_mock.return_value = [rp, other_rp]

        gen_ports = (port for port in
                     [{'id': other_port_id,
                       'fixed_ips': [{'subnet_id': 'other-subnet-id'}],
                       'standard_attr_id': 'standard_attr_id'},
                      {'id': port_id,
                       'fixed_ips': [{'subnet_id': 'subnet-id'}],
                       'standard_attr_id': 'standard_attr_id'}])
        self.fake_l3._plugin = mock.MagicMock()
        self.fake_l3._plugin._make_port_dict = mock.MagicMock(
            side_effect=lambda p: next(gen_ports))

        self.provider._process_before_remove_router_interface('resource',
                                                              'event',
                                                              self,
                                                              payload)

        if port_exists:
            self.fake_l3._ovn_client.create_port.assert_not_called()
            log_mock.assert_not_called()
        else:
            self.fake_l3._ovn_client.create_port.assert_called_once()
            log_mock.assert_called_once()

    def test__process_before_remove_router_interface_port_exists(self):
        self._test__process_before_remove_router_interface(True)

    def test__process_before_remove_router_interface_port_doesnt_exist(self):
        self._test__process_before_remove_router_interface(False)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_concurrency import processutils
from oslo_config import cfg

from neutron import service as neutron_service
from neutron.tests import base


class TestServiceHelpers(base.BaseTestCase):

    def test_get_workers(self):
        num_workers = neutron_service._get_worker_count()
        self.assertGreaterEqual(num_workers, 1)
        self.assertLessEqual(num_workers, processutils.get_worker_count())


class TestRpcWorker(base.BaseTestCase):

    @mock.patch("neutron.policy.refresh")
    @mock.patch("neutron.common.config.setup_logging")
    def _test_reset(self, worker_service, setup_logging_mock, refresh_mock):
        worker_service.reset()

        setup_logging_mock.assert_called_once_with()
        refresh_mock.assert_called_once_with()

    def test_reset(self):
        _plugin = mock.Mock()

        rpc_worker = neutron_service.RpcWorker(_plugin)
        self._test_reset(rpc_worker)


class TestRunRpcWorkers(base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.worker_count = neutron_service._get_worker_count()

    def _test_rpc_workers(self, config_value, expected_passed_value):
        if config_value is not None:
            cfg.CONF.set_override('rpc_workers', config_value)
        with mock.patch('neutron.service.RpcWorker') as mock_rpc_worker:
            with mock.patch('neutron.service.RpcReportsWorker'):
                neutron_service._get_rpc_workers(plugin=mock.Mock())
        init_call = mock_rpc_worker.call_args
        if expected_passed_value > 0:
            expected_call = mock.call(
                mock.ANY, worker_process_count=expected_passed_value)
            self.assertEqual(expected_call, init_call)
        else:
            mock_rpc_worker.assert_not_called()

    def test_rpc_workers_zero(self):
        self._test_rpc_workers(0, 0)

    def test_rpc_workers_default_api_workers_default(self):
        workers = max(int(self.worker_count / 2), 1)
        self._test_rpc_workers(None, workers)

    def test_rpc_workers_default_api_workers_set(self):
        cfg.CONF.set_override('api_workers', 18)
        self._test_rpc_workers(None, 9)

    def test_rpc_workers_defined(self):
        self._test_rpc_workers(42, 42)



# Copyright 2024 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutron_lib import constants
import sqlalchemy as sa

from neutron.db import migration


# Add NUMA policy 'socket'
#
# Revision ID: 175fa80908e1
# Revises: 0e6eff810791
# Create Date: 2024-02-24 10:25:52.418502

# revision identifiers, used by Alembic.
revision = '175fa80908e1'
down_revision = '0e6eff810791'

table = 'portnumaaffinitypolicies'
new_enum = sa.Enum(*constants.PORT_NUMA_POLICIES,
                   name='numa_affinity_policy')


def upgrade():
    migration.alter_enum_add_value(table, 'numa_affinity_policy', new_enum,
                                   True)



# Copyright (c) 2025 Red Hat Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants as lib_consts
from neutron_lib import context
from neutron_lib.exceptions import qos as qos_exc
from neutron_lib.services.qos import constants as qos_consts

from neutron.objects.qos import policy
from neutron.objects.qos import qos_policy_validator
from neutron.objects.qos import rule
from neutron.tests.unit import testlib_api


class TestCheckBandwidthRuleConflict(testlib_api.SqlTestCase):

    def setUp(self):
        super().setUp()
        self.context = context.get_admin_context()
        self.qos_policy = policy.QosPolicy(self.context)
        self.qos_policy.create()
        self.max_bw_egress = 10000
        self.max_bw_rule = rule.QosBandwidthLimitRule(
            self.context, qos_policy_id=self.qos_policy.id,
            max_kbps=self.max_bw_egress,
            direction=lib_consts.EGRESS_DIRECTION)
        self.max_bw_rule.create()
        self.qos_policy.rules = [self.max_bw_rule]

    def test_check_bandwidth_rule_conflict_different_direction(self):
        rule_data = {qos_consts.DIRECTION: lib_consts.INGRESS_DIRECTION,
                     qos_consts.MIN_KBPS: self.max_bw_egress + 1}
        qos_policy_validator.check_bandwidth_rule_conflict(
            self.qos_policy, rule_data)

    def test_check_bandwidth_rule_conflict_same_direction(self):
        rule_data = {qos_consts.DIRECTION: lib_consts.EGRESS_DIRECTION,
                     qos_consts.MIN_KBPS: self.max_bw_egress + 1}
        self.assertRaises(qos_exc.QoSRuleParameterConflict,
                          qos_policy_validator.check_bandwidth_rule_conflict,
                          self.qos_policy, rule_data)



# Copyright 2025 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ovsdbapp.backend.ovs_idl import event as row_event

from neutron.services.bgp import constants


class BGPChassisEvent(row_event.RowEvent):
    TABLE = 'Chassis_Private'

    def __init__(self, reconciler):
        super().__init__(self.EVENTS, self.TABLE, None)
        self.reconciler = reconciler

    @property
    def event_name(self):
        return self.__class__.__name__


class BGPChassisBridgesUpdateEvent(BGPChassisEvent):
    """Event for chassis BGP bridges updates.

    This event is triggered only if bgp-bridges are changed.
    """
    EVENTS = (BGPChassisEvent.ROW_UPDATE,)

    def match_fn(self, event, row, old):
        if not super().match_fn(event, row, old):
            return False
        if not hasattr(old, 'external_ids'):
            return False
        current_bgp_bridges = row.external_ids.get(
            constants.CHASSIS_BGP_BRIDGES_EXT_ID_KEY)
        old_bgp_bridges = old.external_ids.get(
            constants.CHASSIS_BGP_BRIDGES_EXT_ID_KEY)

        return current_bgp_bridges != old_bgp_bridges

    def run(self, event, row, old):
        self.reconciler.reconcile(
            self.reconciler.BGPReconcilerResource.CHASSIS_BGP_BRIDGES,
            row)
